#
#  $Id$
#
#  (C) Copyright 2003 Kai Sterker <kaisterker@linuxgames.com>
#  Part of the Adonthell Project http://adonthell.linuxgames.com
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License.
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY.
#
#  See the COPYING file for more details
#

class damage (object):
    """Class that implements a certain damage type and method. 
    
    There are four damage types corresponding to the four types of 
    resistances (Physical, Fire, Poison and Magic), plus a fifth type 
    that can be used to modify either of the former (especially useful 
    for fighting Feats).
 
    Damage objects that deal the same kind of damage need to have the
    same id, so that their effect can be accumulated. Damage objects
    that should not be accumulated should not get the same id.

    Note that this class is abstract and needs to be subclassed to
    be of any use."""
    
    # -- The different types of damage. These values are also used
    #    to define the priority of damage types. I.e., if some object
    #    causes both Physical and Fire damage, the Fire damage will
    #    always be applied first.
    PHYSICAL = 0
    FIRE = 1
    POISON = 2
    MAGIC = 3
    SPECIAL = 4
    
    def __init__ (self, d_id, d_type, d_dmg):
        """Create a new damage object.
        
        d_id: the Id of the object, used to accumulate similar damage objects
        d_type: the type of damage (Physical, Fire, Poison, Magic, Special)
        d_dmg: the damage the object does, used by the deal_damage method"""
        self.Id = d_id
        self.Type = d_type
        self.Damage = d_dmg
    
    def __cmp__ (self, other):
        """Compare two damage object for their priority."""
        return other.Type - self.Type
    
    def merge (self, other):
        """Accumulate the damage of two similar damage objects. Override if 
        needed."""
        if self.Id == other.Id and self.Type == other.Type:
            self.Damage = (self.Damage[0] + other.Damage[0], self.Damage[1] + other.Damage[1])
            return 1
        else: return 0
        
    def to_string (self):
        """Return a textual representation of the damage object. Must be
        implemented by any subclass."""
        raise NotImplementedError, "damage::to_string"
    
    def apply_effect (self, attacker, defender):
        """Apply the damage effect to a character. Must be implemented by
        any subclass."""
        raise NotImplementedError, "damage::apply_effect"
